# Constants used throughout
WINDOW_HEIGHT = 750
WINDOW_WIDTH = 1500
COLUMN_WIDTH = 150

DISPLAY_REGION_INFO = (
    True  # Display brain region info string in bottom right corner
)

LOADING_PANEL_ALIGN = (
    "left"  # Alignment of text in pushbuttons in loading panel
)
SEGM_METHODS_PANEL_ALIGN = (
    "left"  # Alignment of text in pushbuttons in methods chooser panel
)

POINT_SIZE = 100
SPLINE_SIZE = 50
NUM_COLORS = 10
BRUSH_SIZE = 250
SPLINE_POINTS_DEFAULT = 1000
SPLINE_SMOOTHING_DEFAULT = 0.1
FIT_DEGREE_DEFAULT = 3

SUMMARISE_TRACK_DEFAULT = True
CALCULATE_VOLUMES_DEFAULT = True
SUMMARIZE_VOLUMES_DEFAULT = True

TRACK_FILE_EXT = ".points"
IMAGE_FILE_EXT = ".tiff"
BOUNDARIES_STRING = "Boundaries"

# ORIENTATIONS = [
#     "asl",
#     "lsa",
#     "sla",
# ]  # When rolling through orientations of the brain, adhere to this order
